#!/usr/bin/python

"""
common.py - commonly used scene classes
"""

import os
import pygame
from pygame.locals import *
from gamedirector import *
from math import *

import resources

class FadeInOut(object):
    def __init__(self,ticks,smooth=True):
        self.ticks = ticks
        self.ticks_elapsed = 0
        self.direction = 'none'
        self.alpha = 255
        self.finished_in = False
        self.finished_out = False
        self.musicfade = False
        self.smooth = smooth
        self.running = False
    
    def Update(self):
        if not self.running:
            return
        if self.direction == 'in' and not self.finished_in:
            self.ticks_elapsed += 1
            if self.ticks_elapsed > self.ticks:
                self.finished_in = True
            else:
                if self.smooth:
                    self.alpha = 255*(1.0-(self.ticks_elapsed/float(self.ticks)))
                else:
                    self.alpha = 255*(1.0-((5*(self.ticks_elapsed/5))/float(self.ticks)))
        elif self.direction == 'out' and not self.finished_out:
            self.ticks_elapsed += 1
            if self.ticks_elapsed > self.ticks:
                self.finished_out = True
            else:
                if self.smooth:
                    self.alpha = 255*(self.ticks_elapsed/float(self.ticks))
                else:
                    self.alpha = 255*((5*(self.ticks_elapsed/5))/float(self.ticks))
                if self.musicfade:
                    pygame.mixer.music.set_volume(0.5*(1.0 - (self.ticks_elapsed/float(self.ticks))))
    
    def FadeIn(self):
        self.direction = 'in'
        self.ticks_elapsed = 0
        self.finished_in = False
        self.finished_out = False
        self.running = True
    
    def FadeOut(self,musicfade=False):
        self.direction = 'out'
        self.ticks_elapsed = 0
        self.finished_out = False
        self.finished_in = False
        self.musicfade = musicfade
        self.running = True
    
    def Reset(self):
        self.ticks_elapsed = 0
        self.direction = 'none'
        self.alpha = 255
        self.finished_in = False
        self.finished_out = False
        self.musicfade = False
        self.running = False

class MenuButton(object):
    def __init__(self,pos,rect_on,rect_off,surf_on,surf_off):
        self.pos = pos
        self.hover = False
        self.rect_on = rect_on
        self.rect_off = rect_off
        self.surf_on = surf_on
        self.surf_off = surf_off
    
    def Draw(self, screen):
        if self.hover:
            screen.blit(self.surf_on, self.pos)
        else:
            screen.blit(self.surf_off, self.pos)

class MenuList(object):
    def __init__(self,buttons):
        self.buttons = buttons
        self.currentbutton = None
        self.cursor_pos_screen = pygame.mouse.get_pos()
    
    def SetMousePos(self):
        self.cursor_pos_screen = pygame.mouse.get_pos()
    
    def Reset(self):
        self.currentbutton = None
        self.cursor_pos_screen = pygame.mouse.get_pos()
    
    def SetGoFunc(self,button_ind,func):
        self.buttons[button_ind].go_func = func
    
    def ProcessKeyEvent(self,event):
        if event.type == MOUSEBUTTONDOWN:
            self.SetMousePos()
            if not self.currentbutton == None:
                self.currentbutton.go_func(self.switchtoargs)
        elif event.type == MOUSEMOTION:
            self.SetMousePos()
            self.currentbutton = None
            for button in self.buttons:
                if self.cursor_pos_screen[0] > (button.rect_off.x+button.pos[0]) and self.cursor_pos_screen[0] < (button.rect_off.x+button.rect_off.w+button.pos[0]) and \
                    self.cursor_pos_screen[1] > (button.rect_off.y+button.pos[1]) and self.cursor_pos_screen[1] < (button.rect_off.y+button.rect_off.h+button.pos[1]):
                    button.hover = True
                    self.currentbutton = button
                else:
                    button.hover = False
    
    def Draw(self, screen):
        for b in self.buttons:
            b.Draw(screen)

class ClickOptionsScreen(GameScene):
    def __init__(self, director, window_size, fadelevel):
        super(ClickOptionsScreen, self).__init__(director)
        self.window_size = window_size
        self.cursorsize = 24
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        self.fadelevel = fadelevel
    
    def on_switchto(self, switchtoargs):
        self.switchtoargs = switchtoargs
        self.menulist.switchtoargs = switchtoargs
        self.menulist.Reset()
        
        # Setup background
        if self.fadelevel > 0:
            backfade = pygame.Surface(self.background.get_size())
            backfade.fill((0,0,0))
            backfade.convert()
            backfade.set_alpha(self.fadelevel)
            self.background.blit(backfade, (0,0))
        
    def SetMousePos(self):
        self.cursor_pos_screen = pygame.mouse.get_pos()
        
    def on_update(self):
        pass
    
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                self.EscapeFunc(self.switchtoargs)
            else:
                self.menulist.ProcessKeyEvent(event)
    
    def on_draw(self, screen):
        screen.blit(self.background, (0,0))
        self.menulist.Draw(screen)
        self.SetMousePos()
        tilecoords = resources.coords_targets_sprite['mouse']
        screen.blit(resources.img_targets_sprite, (self.cursor_pos_screen[0],self.cursor_pos_screen[1]), area=tilecoords)

class CutScene(GameScene):
    def __init__(self, director, window_size):
        super(CutScene, self).__init__(director)
        self.window_size = window_size
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        
        self.midground = pygame.Surface(window_size)
        self.midground.fill((0,0,0))
        self.midground.convert()
        
        self.fadebackground = pygame.Surface(window_size)
        self.fadebackground.fill((0,0,0))
        self.fadebackground.convert()
        
        self.fade = FadeInOut(30)
        self.fade_text = FadeInOut(15)
        
        self.go_next = False
        
    def on_switchto(self, switchtoargs):
        
        self.sequence = switchtoargs[0] # [ [bgid, [textid1, textid2, ...] ]
        self.musicid = switchtoargs[1]
        self.next_scene_type = switchtoargs[2]
        self.next_scene_switchtoargs = switchtoargs[3]
        
        self.step = 0
        self.textstep = -1
        
        self.ani_to = 0
        self.ani_frame1 = 0
        self.ani_frame2 = 0
        
        # Check music
        pygame.mixer.music.stop()
        if not self.musicid == 'none':
            pygame.mixer.music.load(resources.musicpaths[self.musicid])
            pygame.mixer.music.set_volume(0.5)
            pygame.mixer.music.play(-1)
        
        self.fade.FadeIn()
        
    def on_update(self):
        
        # Check for return to game
        self.fade.Update()
        self.fade_text.Update()
        if self.fade.direction == 'in' and self.fade.finished_in and not self.fade_text.running:
            if len(self.sequence[self.step][1]) > 0:
                self.textstep = 0
                self.fade_text.FadeIn()
        if self.go_next:
            if len(self.sequence[self.step][1]) > 0: # any text?
                if self.fade.direction == 'in' and self.fade_text.direction == 'in' and self.fade_text.running:
                    self.fade_text.FadeOut()
            else:
                self.fade.FadeOut()
            self.go_next = False
        if self.fade_text.finished_out: # next text
            self.fade_text.Reset()
            self.textstep += 1
            if self.textstep == len(self.sequence[self.step][1]):
                self.textstep = -1
                if self.step == len(self.sequence)-1:
                    self.fade.FadeOut(True)
                else:
                    self.fade.FadeOut()
            else:
                self.fade_text.FadeIn()
        if self.fade.finished_out: # next cut
            self.step += 1
            if self.step == len(self.sequence):
                self.director.change_scene(self.next_scene_type,self.next_scene_switchtoargs)
            else:
                self.fade.Reset()
                self.fade.FadeIn()
        
    def on_event(self, events):
        for event in events:
            if event.type == KEYDOWN and event.key == K_ESCAPE:
                pygame.mixer.music.stop()
                self.director.change_scene(self.next_scene_type,self.next_scene_switchtoargs)
            if event.type == KEYDOWN or event.type == MOUSEBUTTONDOWN:
                self.go_next = True
    
    def render_animation(self, screen):
        raise NotImplementedError("render_animation abstract method must be defined in subclass.")
    
    def render_bg(self, screen):
        bgdata = self.sequence[self.step][0]
        if not bgdata == 'none':
            surf = resources.cutscenesurfs[bgdata]
            pos = ((self.window_size[0]/2)-(surf.get_width()/2), (self.window_size[1]/2)-(surf.get_height()/2))
            screen.blit(surf, pos)
    
    def render_text(self, screen):
        textdata = self.sequence[self.step][1][self.textstep]
        surf = resources.cutscenesurfs[textdata]
        #pos = ((self.window_size[0]/2)-(surf.get_width()/2), (self.window_size[1]/2)-(surf.get_height()/2))
        pos = ((self.window_size[0]/2)-(surf.get_width()/2), 20)
        surf.set_alpha(255-self.fade_text.alpha)
        screen.blit(surf, pos)
    
    def on_draw(self, screen):
        screen.blit(self.background, (0, 0))
        self.render_bg(screen)
        if self.textstep >= 0:
            self.render_text(screen)
        self.fadebackground.set_alpha(self.fade.alpha)
        screen.blit(self.fadebackground, (0, 0))

class TitleScreen(ClickOptionsScreen):    
    
    def __init__(self, director, window_size, fadelevel, savepath):
        super(TitleScreen, self).__init__(director,window_size,fadelevel)
        
        self.savepath = savepath
        
        pos = resources.titlescreen_data['newgamepos']
        rect_on = resources.titlescreen_data['newgame_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.titlescreen_data['newgame_on']
        surf_off = resources.titlescreen_data['newgame_off']
        self.button1 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.titlescreen_data['continuepos']
        rect_on = resources.titlescreen_data['continue_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.titlescreen_data['continue_on']
        surf_off = resources.titlescreen_data['continue_off']
        self.button2 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        # Background
        self.background = pygame.Surface(window_size)
        self.background.fill((0,0,0))
        self.background.convert()
        
        self.fade = FadeInOut(30)
    
    # Check music
    def on_switchto(self, switchtoargs):
        
        if os.path.isfile(self.savepath):
            self.menulist = MenuList([self.button1,self.button2])
            self.menulist.SetGoFunc(0,self.GoFunc1)
            self.menulist.SetGoFunc(1,self.GoFunc2)
        else:
            self.menulist = MenuList([self.button1])
            self.menulist.SetGoFunc(0,self.GoFunc1)
            
        super(TitleScreen, self).on_switchto(switchtoargs)
        pygame.mixer.music.stop()
        pygame.mixer.music.load(resources.musicpaths['title'])
        pygame.mixer.music.set_volume(1.0)
        pygame.mixer.music.play(-1)
        self.timer = 0
        self.bg_move = 0
        
        self.leaving = False
        self.type = None
        
        self.fade.FadeIn()
    
    def EscapeFunc(self,switchtoargs):
        self.director.change_scene(None, [])
    
    def GoFunc1(self, switchtoargs):
        self.leaving = True
        self.type = 'newgame'
    
    def GoFunc2(self, switchtoargs):
        self.leaving = True
        self.type = 'continue'
    
    def on_update(self):
        
        self.fade.Update()
        if self.leaving and self.fade.direction == 'in':
            musicfade = True
            self.fade.FadeOut(musicfade)
        if self.fade.finished_out:
            if self.type == 'newgame':
                self.h_progress_data.Reset()
                self.director.change_scene('introcutscene', resources.introcutscene_data)
            elif self.type == 'continue':
                self.h_progress_data.LoadProgressData()
                self.director.change_scene('maingame', [True,self.h_progress_data.current_level])
        
        self.timer += 1
        self.bg_move = (self.bg_move+1) % 1280
    
    def on_draw(self, screen):
        screen.blit(self.background, (0,0))
        if self.bg_move >= 640:
            coords1 = (self.bg_move,0,1280-self.bg_move,480)
            screen.blit(resources.titlescreen_data['titlebg'], (0,0), area=coords1)
            coords2 = (0,0,self.bg_move-640,480)
            screen.blit(resources.titlescreen_data['titlebg'], (640-(self.bg_move-640),0), area=coords2)
        else:
            coords1 = (self.bg_move,0,640,480)
            screen.blit(resources.titlescreen_data['titlebg'], (0,0), area=coords1)
        screen.blit(resources.titlescreen_data['title_text'], (0,0))
        if self.timer < 90:
            alpha = 0
        elif self.timer > 150:
            alpha = 255
        else:
            alpha = (255/60)*(self.timer-90)
        self.menulist.buttons[0].surf_on.set_alpha(alpha)
        self.menulist.buttons[0].surf_off.set_alpha(alpha)
        if len(self.menulist.buttons) > 1:
            self.menulist.buttons[1].surf_on.set_alpha(alpha)
            self.menulist.buttons[1].surf_off.set_alpha(alpha)
        self.menulist.Draw(screen)
        self.SetMousePos()
        tilecoords = resources.coords_targets_sprite['mouse']
        screen.blit(resources.img_targets_sprite, (self.cursor_pos_screen[0],self.cursor_pos_screen[1]), area=tilecoords)
        
        self.background.set_alpha(self.fade.alpha)
        screen.blit(self.background, (0, 0))

class PauseGame(ClickOptionsScreen):    
    
    def __init__(self, director, window_size, fadelevel):
        super(PauseGame, self).__init__(director,window_size,fadelevel)
        
        pos = resources.pausemenu_data['continuepos']
        rect_on = resources.pausemenu_data['continue_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['continue_on']
        surf_off = resources.pausemenu_data['continue_off']
        self.button1 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.pausemenu_data['quitpos']
        rect_on = resources.pausemenu_data['quit_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['quit_on']
        surf_off = resources.pausemenu_data['quit_off']
        self.button2 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.pausemenu_data['resetpos']
        rect_on = resources.pausemenu_data['reset_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['reset_on']
        surf_off = resources.pausemenu_data['reset_off']
        self.button3 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.pausemenu_data['positivepos']
        rect_on = resources.pausemenu_data['positive_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['positive_on']
        surf_off = resources.pausemenu_data['positive_off']
        self.button4 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        pos = resources.pausemenu_data['negativepos']
        rect_on = resources.pausemenu_data['negative_on'].get_rect()
        rect_off = rect_on
        surf_on = resources.pausemenu_data['negative_on']
        surf_off = resources.pausemenu_data['negative_off']
        self.button5 = MenuButton(pos,rect_on,rect_off,surf_on,surf_off)
        
        self.menulist = MenuList([self.button1,self.button2,self.button3,self.button4,self.button5])
        self.menulist.SetGoFunc(0,self.GoFunc1)
        self.menulist.SetGoFunc(1,self.GoFunc2)
        self.menulist.SetGoFunc(2,self.GoFunc3)
        self.menulist.SetGoFunc(3,self.GoFunc4)
        self.menulist.SetGoFunc(4,self.GoFunc5)
    
    def EscapeFunc(self,switchtoargs):
        resources.ResumeVoice()
        pygame.mouse.set_pos([320,240])
        self.director.change_scene('maingame', [False, switchtoargs])
    
    def GoFunc1(self, switchtoargs):
        resources.ResumeVoice()
        pygame.mouse.set_pos([320,240])
        self.director.change_scene('maingame', [False, switchtoargs])
    
    def GoFunc2(self, switchtoargs):
        #self.director.change_scene(None, [])
        resources.StopVoice()
        self.maingame.current_music = 'none'
        self.director.change_scene('titlescreen', [])
    
    def GoFunc3(self, switchtoargs):
        resources.StopVoice()
        pygame.mouse.set_pos([320,240])
        self.director.change_scene('maingame', [True, switchtoargs])
    
    def GoFunc4(self, switchtoargs):
        self.maingame.mouse_sensitivity += 0.001
        if self.maingame.mouse_sensitivity > self.maingame.motion.mouse_sensitivity_max:
            self.maingame.mouse_sensitivity = self.maingame.motion.mouse_sensitivity_max
    
    def GoFunc5(self, switchtoargs):
        self.maingame.mouse_sensitivity -= 0.001
        if self.maingame.mouse_sensitivity < self.maingame.motion.mouse_sensitivity_min:
            self.maingame.mouse_sensitivity = self.maingame.motion.mouse_sensitivity_min
    
    def on_draw(self, screen):
        screen.blit(self.background, (0,0))
        self.menulist.Draw(screen)
        
        # Render sensitivity bar
        screen.blit(resources.pausemenu_data['adjustmouse'], resources.pausemenu_data['adjustmousepos'])
        max = self.maingame.motion.mouse_sensitivity_max
        min = self.maingame.motion.mouse_sensitivity_min
        bl = 200*((self.maingame.mouse_sensitivity-min)/(max-min))
        pygame.draw.rect(screen,(128,128,128),(220,340,bl,10),0)
        pygame.draw.rect(screen,(200,200,200),(220,340,200,10),4)
        
        self.SetMousePos()
        tilecoords = resources.coords_targets_sprite['mouse']
        screen.blit(resources.img_targets_sprite, (self.cursor_pos_screen[0],self.cursor_pos_screen[1]), area=tilecoords)

