#!/usr/bin/python

"""
model.py - functions to deal with triangulated models
"""

import os, sys
import pygame
from pygame.locals import *
import numpy as np
from math import *

class Object3d(object):
    def __init__(self,model,pos):
        self.model = model
        self.pos = np.array(pos)
        self.DCM = np.array([[1.0,0.0,0.0],[0.0,1.0,0.0],[0.0,0.0,1.0]])
        self.glow = 0
        
    def SetTransform(self, camera):
        vertex_w = np.zeros(self.model.vertex.shape)
        for i in xrange(self.model.vertex.shape[0]):
            vertex_w[i] = np.dot(self.model.vertex[i],self.DCM)
        self.normal_w = np.zeros(self.model.normal.shape)
        for i in xrange(self.model.normal.shape[0]):
            self.normal_w[i] = np.dot(self.model.normal[i],self.DCM)
        self.vertex_c = vertex_w+self.pos-camera.pos
        self.depth = np.dot(self.vertex_c,camera.DCM[0])
        self.x = camera.f*np.dot(self.vertex_c,camera.DCM[1])/self.depth + camera.uv0[0]
        self.y = camera.f*np.dot(self.vertex_c,camera.DCM[2])/self.depth + camera.uv0[1]

class Light(object):
    def __init__(self,pos,colour,type='distant',radius=(1.0,50.0)):
        self.pos = np.array(pos)
        self.colour = colour
        self.type = type
        if self.type == 'point':
            self.radius = radius

class Camera(object):
    def __init__(self,screen_size,pos):
        self.w = screen_size[0]
        self.h = screen_size[1]
        self.pos = np.array(pos)
        self.DCM = np.array([[1.0,0.0,0.0],[0.0,1.0,0.0],[0.0,0.0,1.0]])
        self.f = 0.5*screen_size[0]
        self.uv0 = (screen_size[0]/2,screen_size[1]/2)
        
        self.radius = 2.0
        
        self.objects = []
        self.lights = []
        
        self.amb_strength = 0.5
        self.amb_colour = [1.0,1.0,1.0]
    
    def AddObject3d(self,object3d):
        self.objects.append(object3d)
    
    def AddLight(self,light):
        self.lights.append(light)
    
    def DrawScene(self,screen):
        self.facedata = []
        for m in self.objects:
            m.SetTransform(self)
            for i in xrange(len(m.model.face)):
                f = m.model.face[i]
                if m.depth[f[0]] == 0.0 or m.depth[f[1]] == 0.0 or m.depth[f[2]] == 0.0:
                    continue
                verts = [[m.x[f[0]],m.y[f[0]]],[m.x[f[1]],m.y[f[1]]],[m.x[f[2]],m.y[f[2]]]]
                v1_in = verts[0][0] > 0.0 and verts[0][0] < self.w and verts[0][1] > 0.0 and verts[0][1] < self.h and m.depth[f[0]] > 0.0
                v2_in = verts[1][0] > 0.0 and verts[1][0] < self.w and verts[1][1] > 0.0 and verts[1][1] < self.h and m.depth[f[1]] > 0.0
                v3_in = verts[2][0] > 0.0 and verts[2][0] < self.w and verts[2][1] > 0.0 and verts[2][1] < self.h and m.depth[f[2]] > 0.0
                if v1_in or v2_in or v3_in:
                    avg_depth = (m.depth[f[0]]+m.depth[f[1]]+m.depth[f[2]])/3.0
                    shade = [0.0,0.0,0.0]
                    
                    shade[0] += self.amb_strength*m.model.colour[i][0]*self.amb_colour[0]
                    shade[1] += self.amb_strength*m.model.colour[i][1]*self.amb_colour[1]
                    shade[2] += self.amb_strength*m.model.colour[i][2]*self.amb_colour[2]
                    
                    for l in self.lights:
                        if l.type == 'point':
                            mpdiff = m.model.vertex[f[0]]-l.pos
                            r = np.linalg.norm(mpdiff)
                            if r <= l.radius[0]:
                                strength = 1.0
                            elif r >= l.radius[1]:
                                strength = 0.0
                            else:
                                strength = (l.radius[1]-r)/(l.radius[1]-l.radius[0])
                        elif l.type == 'distant':
                            mpdiff = -l.pos
                            strength = 1.0
                        mpdiff = mpdiff/np.linalg.norm(mpdiff)
                        ctheta = np.dot(m.normal_w[i],mpdiff)
                        #ctheta = np.dot(m.model.normal[i],mpdiff)
                        #shade[0] += max(min(strength*ctheta*m.model.colour[i][0]*l.colour[0],1.0),0.0)
                        #shade[1] += max(min(strength*ctheta*m.model.colour[i][1]*l.colour[1],1.0),0.0)
                        #shade[2] += max(min(strength*ctheta*m.model.colour[i][2]*l.colour[2],1.0),0.0)
                        shade[0] += strength*ctheta*m.model.colour[i][0]*l.colour[0]
                        shade[1] += strength*ctheta*m.model.colour[i][1]*l.colour[1]
                        shade[2] += strength*ctheta*m.model.colour[i][2]*l.colour[2]
                    #print sum(m.normal[i]), sum(mpdiff), np.dot(m.normal[i],mpdiff), shade
                    
                    shade[0] += 0.05*m.glow
                    shade[1] += 0.05*m.glow
                    shade[2] += 0.05*m.glow
                    
                    shade[0] = max(min(shade[0],1.0),0.0)
                    shade[1] = max(min(shade[1],1.0),0.0)
                    shade[2] = max(min(shade[2],1.0),0.0)
                    #print shade
                    self.facedata.append(verts+[avg_depth,shade])
        for p in self.projectiles.p:
            if p.active:
                depth1 = np.dot(p.pos-self.pos,self.DCM[0])
                x1 = self.f*np.dot(p.pos-self.pos,self.DCM[1])/depth1 + self.uv0[0]
                y1 = self.f*np.dot(p.pos-self.pos,self.DCM[2])/depth1 + self.uv0[1]
                if depth1 > 0.0:
                    radius = 60.0*(1.0/depth1)
                    if radius > 20.0:
                        radius = 20.0
                    elif radius < 1.0:
                        radius = 1.0
                    self.facedata.append([x1,y1,[radius,p.type],depth1])
        if self.player.alive and self.motiondust.active:
            for i in xrange(len(self.motiondust.x1)):
                if self.motiondust.depth1[i] > 0.0 and self.motiondust.depth2[i] > 0.0 and self.motiondust.range1[i] < (self.motiondust.gridsize/2):
                    self.facedata.append([self.motiondust.x1[i],self.motiondust.y1[i],[self.motiondust.x2[i],self.motiondust.y2[i],0],self.motiondust.depth1[i]])
                
        self.facedata.sort(reverse=True,key=lambda facedata: facedata[3])
        for face in self.facedata:
            if len(face) == 5:
                verts = face[0:3]
                colour = face[4]
                pygame.draw.polygon(screen, (255*face[4][0],255*face[4][1],255*face[4][2]), verts, 0)
            else:
                x1 = face[0]
                y1 = face[1]
                if len(face[2]) == 2:
                    radius = face[2][0]
                    type = face[2][1]
                    if type == 'laser1' or type == 'laser4':
                        pygame.draw.circle(screen, (100,0,0), (int(x1),int(y1)), int(radius), 0)
                        pygame.draw.circle(screen, (255,0,0), (int(x1),int(y1)), int(ceil(radius/2)), 0)
                    elif type == 'laser2':
                        pygame.draw.circle(screen, (0,100,0), (int(x1),int(y1)), int(radius), 0)
                        pygame.draw.circle(screen, (0,255,0), (int(x1),int(y1)), int(ceil(radius/2)), 0)
                    elif type == 'laser3':
                        pygame.draw.circle(screen, (100,0,100), (int(x1),int(y1)), int(radius), 0)
                        pygame.draw.circle(screen, (255,0,255), (int(x1),int(y1)), int(ceil(radius/2)), 0)
                else:
                    x2 = face[2][0]
                    y2 = face[2][1]
                    pygame.draw.line(screen, (255,255,255), (x1,y1), (x2,y2))
                    
        
        
        