#!/usr/bin/python

"""
spacecraft.py - functions to deal with player and other spacecraft
"""

import os, sys
import pygame
from pygame.locals import *
import numpy as np
from math import *
import random

import resources
import model

class MotionControl(object):
    def __init__(self,parent,controlmap,camera,player):
        self.parent = parent
        self.CM = controlmap
        self.camera = camera
        self.player = player
        
        self.u = 0
        self.d = 0
        self.l = 0
        self.r = 0
        self.fire = False
        
        self.pitch = 0.0
        self.yaw = 0.0
        
        self.velocity = [0.0,0.0,0.0]
        self.accel = 0.05
        #self.maxv = 0.5
        self.maxv = 0.5
        self.driftdecel = 0.005
        self.driftvel = 0.05
        
        self.mouse_sensitivity_min = 0.001
        self.mouse_sensitivity_max = 0.01
        
        self.mouse_rel_events = []
        self.mouse_rel_ignore = [1000,1000]
    
    def ProcessKeyEvent(self,event):
        if self.parent.warping_out:
            self.u = 0
            self.d = 0
            self.l = 0
            self.r = 0
        if event.type == KEYDOWN:
            if not self.player.alive:
                self.parent.leaving_mission = True
            else:
                if event.key == self.CM["U"]:
                    self.u = 1
                elif event.key == self.CM["D"]:
                    self.d = 1
                elif event.key == self.CM["L"]:
                    self.l = 1
                elif event.key == self.CM["R"]:
                    self.r = 1
                elif event.key == self.CM["Fire"]:
                    if self.player.alive:
                        vproj = 3.0*np.dot(np.array([1.0,0.0,0.0]),self.camera.DCM)
                        offset = np.dot(np.array([0.0,0.0,0.5]),self.camera.DCM)
                        #print self.camera.pos
                        self.parent.projectiles.New('laser1',self.camera.pos+offset,self.velocity+vproj)
                        self.fire = True
                        #resources.soundfx['laser'].play()
                        resources.PlaySound('laser')
                        #print "fire"
                elif event.key == self.CM["Objective"]:
                    if self.parent.hud.objective_mode:
                        self.parent.hud.objective_mode = False
                    else:
                        self.parent.hud.objective_mode = True
                elif event.key == self.CM["Hostile"]:
                    self.parent.hud.enemy_num += 1
                    while True:
                        if self.parent.hud.enemy_num >= len(self.parent.hud.enemylist):
                            self.parent.hud.enemy_num = -1
                            break
                        if self.parent.hud.enemylist[self.parent.hud.enemy_num].alive:
                            break
                        self.parent.hud.enemy_num += 1
        elif event.type == MOUSEBUTTONDOWN:
            if self.player.alive:
                vproj = 3.0*np.dot(np.array([1.0,0.0,0.0]),self.camera.DCM)
                offset = np.dot(np.array([0.0,0.0,0.5]),self.camera.DCM)
                #print self.camera.pos
                self.parent.projectiles.New('laser1',self.camera.pos+offset,self.velocity+vproj)
                self.fire = True
                #resources.soundfx['laser'].play()
                resources.PlaySound('laser')
                #print "fire"
            else:
                self.parent.leaving_mission = True
        elif event.type == KEYUP:
            if event.key == self.CM["U"]:
                self.u = 0
            elif event.key == self.CM["D"]:
                self.d = 0
            elif event.key == self.CM["L"]:
                self.l = 0
            elif event.key == self.CM["R"]:
                self.r = 0
            elif event.key == self.CM["Fire"]:
                self.fire = False
    
    def ProcessMouseMotion(self,event):
        #print event.rel, self.mouse_rel_ignore
        if not event.rel == self.mouse_rel_ignore:
            self.mouse_rel_events.append(event.rel)
        
    def UpdateMotion(self):
        if self.player.alive:
            dvx = self.accel*(self.u-self.d)
            dvy = self.accel*(self.r-self.l)
            self.velocity[0] += self.camera.DCM[0][0]*dvx + self.camera.DCM[1][0]*dvy
            self.velocity[1] += self.camera.DCM[0][1]*dvx + self.camera.DCM[1][1]*dvy
            self.velocity[2] += self.camera.DCM[0][2]*dvx + self.camera.DCM[1][2]*dvy
            
            totalv = sqrt(pow(self.velocity[0],2) + pow(self.velocity[1],2) + pow(self.velocity[2],2))
            if totalv > self.maxv:
                self.velocity[0] = self.maxv*(self.velocity[0]/totalv)
                self.velocity[1] = self.maxv*(self.velocity[1]/totalv)
                self.velocity[2] = self.maxv*(self.velocity[2]/totalv)
            if self.u == 0 and self.d == 0 and self.l == 0 and self.r == 0 and totalv > 0:
                self.velocity[0] -= self.driftdecel*(self.velocity[0]/totalv)
                self.velocity[1] -= self.driftdecel*(self.velocity[1]/totalv)
                self.velocity[2] -= self.driftdecel*(self.velocity[2]/totalv)
                totalv = sqrt(pow(self.velocity[0],2) + pow(self.velocity[1],2) + pow(self.velocity[2],2))
                if totalv < self.driftvel:
                    self.velocity[0] = self.driftvel*(self.velocity[0]/totalv)
                    self.velocity[1] = self.driftvel*(self.velocity[1]/totalv)
                    self.velocity[2] = self.driftvel*(self.velocity[2]/totalv)
            self.camera.pos[0] += self.velocity[0]
            self.camera.pos[1] += self.velocity[1]
            self.camera.pos[2] += self.velocity[2]
            
            #self.camera.pos[0] += self.camera.DCM[0][0]*dx + self.camera.DCM[1][0]*dy
            #self.camera.pos[1] += self.camera.DCM[0][1]*dx + self.camera.DCM[1][1]*dy
            #self.camera.pos[2] += self.camera.DCM[0][2]*dx + self.camera.DCM[1][2]*dy
            if not self.parent.warping_out:
                rel = [self.parent.mp[0]-320,self.parent.mp[1]-240]
                self.pitch -= self.parent.mouse_sensitivity*rel[1]
                if self.pitch >= pi/2:
                    self.pitch = pi/2-0.0001
                elif self.pitch <= -pi/2:
                    self.pitch = -pi/2+0.0001
                self.yaw -= self.parent.mouse_sensitivity*rel[0]
                self.camera.DCM = np.array([[cos(self.pitch)*cos(self.yaw),-cos(self.pitch)*sin(self.yaw),-sin(self.pitch)],\
                    [sin(self.yaw),cos(self.yaw),0.0],[sin(self.pitch)*cos(self.yaw),-sin(self.pitch)*sin(self.yaw),cos(self.pitch)]])
                """
                for rel in self.mouse_rel_events:
                    #print rel, self.mouse_rel_ignore
                    self.pitch -= self.parent.mouse_sensitivity*rel[1]
                    if self.pitch >= pi/2:
                        self.pitch = pi/2-0.0001
                    elif self.pitch <= -pi/2:
                        self.pitch = -pi/2+0.0001
                    self.yaw -= self.parent.mouse_sensitivity*rel[0]
                    self.camera.DCM = np.array([[cos(self.pitch)*cos(self.yaw),-cos(self.pitch)*sin(self.yaw),-sin(self.pitch)],\
                        [sin(self.yaw),cos(self.yaw),0.0],[sin(self.pitch)*cos(self.yaw),-sin(self.pitch)*sin(self.yaw),cos(self.pitch)]])
                """
        self.mouse_rel_events = []
        #print "pitch: ", self.pitch, " yaw: ", self.yaw

class HUD(object):
    def __init__(self,parent,window_size,player):
        self.parent = parent
        self.player = player
        self.hitglow = pygame.Surface(window_size)
        self.hitglow.fill((255,255,128))
        self.hitglow.convert()
        
        self.enemy_num = -1
        self.enemylist = []
        self.objective_mode = False
    
    def RenderTarget(self,screen,vertex_c,camera,depth,x,y,type):
        if (depth > 0.0) and (x >= 0) and (x < 640) and (y >= 0) and (y < 480):
            screen.blit(resources.img_targets_sprite, (x-16,y-16), \
                area=resources.coords_targets_sprite['frame'][type])
        else:
            vertex_c2 = [np.dot(vertex_c,camera.DCM[0]),np.dot(vertex_c,camera.DCM[1]),np.dot(vertex_c,camera.DCM[2])]
            hordist = sqrt(vertex_c2[0]*vertex_c2[0] + vertex_c2[1]*vertex_c2[1])
            if hordist > 0.0:
                azimuth = atan2(vertex_c2[1],vertex_c2[0])
                elev = atan(vertex_c2[2]/hordist)
                if (depth <= 0.0):
                    y = 320*tan(elev) + 240
                    if y < 16:
                        y = 16
                        direction = 1
                    elif y > 480-16:
                        y = 480-16
                        direction = 3
                    if azimuth < 0:
                        x = 0+16
                        direction = 0
                    else:
                        x = 640-16
                        direction = 2
                else:
                    if x < 16:
                        x = 16
                        y = 320*tan(elev) + 240
                        direction = 0
                    elif x > 640-16:
                        x = 640-16
                        y = 320*tan(elev) + 240
                        direction = 2
                    else:
                        direction = -1
                    if y < 16:
                        y = 16
                        if direction == -1:
                            direction = 1
                    elif y > 480-16:
                        y = 480-16
                        if direction == -1:
                            direction = 3
                screen.blit(resources.img_targets_sprite, (x-16,y-16), \
                    area=resources.coords_targets_sprite['arrow'][type][direction])
    
    def Draw(self,screen):
        if self.player.alive:
            camera = self.player.camera
            screen.blit(resources.img_targets_sprite, (320-16,240-16), area=resources.coords_targets_sprite['reticule'])
            
            # render shields
            screen.blit(resources.HUD_surfs['shields'],(20,430))
            bl = 100*(self.player.health/10.0)
            if bl > 50:
                bcolour = (0,180,0)
            elif bl > 25:
                bcolour = (180,180,0)
            else:
                bcolour = (180,0,0)
            pygame.draw.rect(screen,bcolour,(20,450,bl,10),0)
            pygame.draw.rect(screen,(128,128,128),(20,450,100,10),4)
            
            # render objectives descriptor
            if not self.parent.mission.finished_obj:
                screen.blit(resources.HUD_surfs['obj'],(180,440))
                screen.blit(resources.HUD_surfs['objectives'][self.parent.mission.name][self.parent.mission.current_objective],(255,440))
            
            # Render targets (hostiles)
            if self.enemy_num > -1:
                s = self.enemylist[self.enemy_num]
                if s.alive:
                    vertex_c = s.pos-camera.pos
                    depth = np.dot(vertex_c,camera.DCM[0])
                    x = camera.f*np.dot(vertex_c,camera.DCM[1])/depth + camera.uv0[0]
                    y = camera.f*np.dot(vertex_c,camera.DCM[2])/depth + camera.uv0[1]
                    self.RenderTarget(screen,vertex_c,camera,depth,x,y,'enemy')
                else:
                    self.enemy_num = -1
            
            # render objectives
            if self.objective_mode and not self.parent.mission.finished_obj:
                if self.parent.mission.obj_loc[0] == 'ships':
                    type = 'shiplist'
                else:
                    type = 'other'
                if type == 'shiplist':
                    for obj in self.parent.mission.obj_loc:
                        if obj == 'ships':
                            continue
                        s = self.parent.spacecraft[obj]
                        if s.alive:
                            vertex_c = s.pos-camera.pos
                            depth = np.dot(vertex_c,camera.DCM[0])
                            x = camera.f*np.dot(vertex_c,camera.DCM[1])/depth + camera.uv0[0]
                            y = camera.f*np.dot(vertex_c,camera.DCM[2])/depth + camera.uv0[1]
                            self.RenderTarget(screen,vertex_c,camera,depth,x,y,'objective')
                        else:
                            self.enemy_num = -1
                elif type == 'other':
                    vertex_c = self.parent.mission.obj_loc-camera.pos
                    depth = np.dot(vertex_c,camera.DCM[0])
                    x = camera.f*np.dot(vertex_c,camera.DCM[1])/depth + camera.uv0[0]
                    y = camera.f*np.dot(vertex_c,camera.DCM[2])/depth + camera.uv0[1]
                    self.RenderTarget(screen,vertex_c,camera,depth,x,y,'objective')
            
            # Screen glow from player hit
            if self.player.glow > 0:
                self.hitglow.set_alpha(8*self.player.glow)
                screen.blit(self.hitglow, (0, 0))
            if self.parent.warping_out:
                screen.blit(resources.HUD_surfs['jump'],(320-resources.HUD_surfs['jump'].get_width()/2,220))
        else:
            screen.blit(resources.HUD_surfs['failed'],(255,220))
                    
class Player(object):
    def __init__(self,parent,camera):
        self.parent = parent
        self.camera = camera
        self.alive = True
        self.health = 10
        self.radius = 1.0
        self.glow = 0
    
    def Update(self):
        if self.glow > 0:
            self.glow -= 1
    
    def Hit(self,damage):
        if self.alive:
            if self.parent.warping_out:
                pass
            else:
                self.glow = 10
                self.health -= damage
                if self.health <= 3 and (self.health+damage) > 3:
                    resources.PlaySound('shields_low')
                if self.health <= 0:
                    self.alive = False
                    self.parent.explosioneffects.NewExplosion(self.camera.pos,np.array([0.0,0.0,0.0]))
                    self.camera.pos = self.camera.pos-[5.0,0.0,0.0]
                    pitch = 0.0
                    yaw = 0.0
                    self.camera.DCM = np.array([[cos(pitch)*cos(yaw),-cos(pitch)*sin(yaw),-sin(pitch)],\
                        [sin(yaw),cos(yaw),0.0],[sin(pitch)*cos(yaw),-sin(pitch)*sin(yaw),cos(pitch)]])
                    #resources.soundfx['explode'].play()
                    resources.StopVoice()
                    resources.PlaySound('explode')

class Projectiles(object):
    class Projectile(object):
        def __init__(self):
            self.active = False
            self.type = 'none'
            self.pos = np.array([0.0,0.0,0.0])
            self.vel = np.array([0.0,0.0,0.0])
            self.to = 0
    def __init__(self,camera,N=20):
        self.camera = camera
        self.p = []
        for i in xrange(N):
            self.p.append(self.Projectile())
    def New(self,type,pos,vel):
        for p in self.p:
            if not p.active:
                p.active = True
                p.type = type
                p.pos = pos
                p.vel = vel
                if p.type == 'laser1':
                    p.to = 40
                elif p.type == 'laser2':
                    p.to = 70
                elif p.type == 'laser3':
                    p.to = 90
                elif p.type == 'laser4':
                    p.to = 40
                else:
                    p.to = 1
                break
    def Update(self):
        for p in self.p:
            if p.active:
                p.pos[0] += p.vel[0]
                p.pos[1] += p.vel[1]
                p.pos[2] += p.vel[2]
                p.to -= 1
                if p.to == 0:
                    p.active = False
                else:
                    if p.type == 'laser1':
                        for s in self.spacecraft:
                            if s.alive:
                                posprev = p.pos-p.vel
                                dist = np.linalg.norm(np.cross(s.pos-posprev,s.pos-p.pos))/np.linalg.norm(p.vel)
                                t = np.dot((posprev-s.pos),p.vel)/pow(np.linalg.norm(p.vel),2)
                                #dist = np.linalg.norm(s.pos-p.pos)
                                if dist <= s.radius and t >= 0 and t <= 1.0:
                                    pinter = posprev + t*p.vel
                                    #self.explosioneffects.NewSpark(p.pos)
                                    self.explosioneffects.NewSpark(pinter)
                                    p.active = False
                                    s.Hit(2)
                    else:
                        posprev = p.pos-p.vel
                        dist = np.linalg.norm(np.cross(self.camera.pos-posprev,self.camera.pos-p.pos))/np.linalg.norm(p.vel)
                        t = np.dot((posprev-self.camera.pos),p.vel)/pow(np.linalg.norm(p.vel),2)
                        #dist = np.linalg.norm(s.pos-p.pos)
                        if dist <= self.camera.radius and t >= 0 and t <= 1.0:
                            pinter = posprev + t*p.vel
                            p.active = False
                            if p.type == 'laser2':
                                damage = 2
                            elif p.type == 'laser3':
                                damage = 3
                            elif p.type == 'laser4':
                                damage = 2
                            self.camera.player.Hit(damage)
    """   
    def Draw(self,screen):
        for p in self.p:
            if p.active:
                depth1 = np.dot(p.pos-self.camera.pos,self.camera.DCM[0])
                x1 = self.camera.f*np.dot(p.pos-self.camera.pos,self.camera.DCM[1])/depth1 + self.camera.uv0[0]
                y1 = self.camera.f*np.dot(p.pos-self.camera.pos,self.camera.DCM[2])/depth1 + self.camera.uv0[1]
                if depth1 > 0.0:
                    radius = 60.0*(1.0/depth1)
                    if radius > 20.0:
                        radius = 20.0
                    elif radius < 1.0:
                        radius = 1.0
                    if p.type == 'laser1':
                        pygame.draw.circle(screen, (100,0,0), (int(x1),int(y1)), int(radius), 0)
                        pygame.draw.circle(screen, (255,0,0), (int(x1),int(y1)), int(ceil(radius/2)), 0)
                    elif p.type == 'laser2':
                        pygame.draw.circle(screen, (0,100,0), (int(x1),int(y1)), int(radius), 0)
                        pygame.draw.circle(screen, (0,255,0), (int(x1),int(y1)), int(ceil(radius/2)), 0)
    """

class Debris(object):
    def __init__(self,camera):
        self.camera = camera
        self.objects = []
    
    def AddDebris(self,pos,vel):
        object = model.Object3d(resources.model_debris,pos)
        self.objects.append([object,vel,random.random(),random.random(),random.random()])
        self.camera.AddObject3d(object)
    
    def Update(self):
        for d in self.objects:
            d[0].pos += d[1]
            d[2] += 0.1
            d[3] += 0.1
            d[4] += 0.1
            d[0].DCM = np.array([[cos(d[3])*cos(d[4]),-cos(d[3])*sin(d[4]),-sin(d[3])],\
                [sin(d[4]),cos(d[4]),0.0],[sin(d[3])*cos(d[4]),-sin(d[3])*sin(d[4]),cos(d[3])]])

class Spacecraft(object):
    def __init__(self,parent,camera,pos,type,label=None,id=None):
        self.parent = parent
        self.camera = camera
        self.pos = pos
        self.vel = np.array([0.0,0.0,0.0])
        
        self.type = type
        self.label = label
        self.id = id
        
        if type == 'nav_beacon':
            self.radius = 1.0
            self.pitch = 0.0
            self.roll = 0.0
            self.yaw = 0.0
            self.object = model.Object3d(resources.model_navbeacon,self.pos)
            self.health = 5
        if type == 'nav_beacon_green':
            self.radius = 1.0
            self.pitch = 0.0
            self.roll = 0.0
            self.yaw = 0.0
            self.object = model.Object3d(resources.model_navbeacongreen,self.pos)
            self.health = 5
        if type == 'comm_sat':
            self.radius = 1.0
            self.pitch = random.random()
            self.roll = 0.0
            self.yaw = 0.0
            self.object = model.Object3d(resources.model_comm_sat,self.pos)
            self.health = 10
        if type == 'defence_plat_a':
            self.hostile = True
            self.radius = 2.5
            self.pitch = 0.0
            self.roll = 0.0
            self.yaw = 0.0
            self.object = model.Object3d(resources.model_defence_plat_a,self.pos)
            self.health = 20
            self.attack_range = 100.0
            self.fire_rate = 30
            if self.label == None:
                self.label = 'hostile'
        if type == 'defence_plat_b':
            self.hostile = True
            self.radius = 2.5
            self.pitch = 0.0
            self.roll = 0.0
            self.yaw = 0.0
            self.object = model.Object3d(resources.model_defence_plat_b,self.pos)
            self.health = 40
            self.attack_range = 160.0
            self.fire_rate = 20
            if self.label == None:
                self.label = 'hostile'
        if type == 'interceptor':
            self.hostile = True
            self.radius = 2.5
            self.pitch = 0.0
            self.roll = 0.0
            self.yaw = 0.0
            
            self.velocity = [0.0,0.0,0.0]
            self.accel = 0.05
            #self.maxv = 0.5
            self.maxv = 0.4
            self.driftdecel = 0.005
            self.driftvel = 0.01
            
            self.object = model.Object3d(resources.model_interceptor,self.pos)
            self.health = 20
            self.attack_range = 120.0
            self.fire_rate = 20
            if self.label == None:
                self.label = 'hostile'
            self.viewrange = 250.0
            self.behave = 'attack'
            self.evade_to = 0
        if type == 'transport':
            self.radius = 7.0
            self.pitch = 0.0
            self.roll = 0.0
            self.yaw = 2.6
            self.object = model.Object3d(resources.model_transport,self.pos)
            self.health = 50
        if type == 'station':
            self.radius = 15.0
            self.pitch = 0.0
            self.roll = 0.0
            self.yaw = pi/2
            self.object = model.Object3d(resources.model_station,self.pos)
            self.health = 50
        
        self.pos = [self.object.pos[0],self.object.pos[1],self.object.pos[2]]
        self.object.DCM = np.array([[cos(self.pitch)*cos(self.yaw),-cos(self.pitch)*sin(self.yaw),-sin(self.pitch)],\
            [sin(self.yaw),cos(self.yaw),0.0],[sin(self.pitch)*cos(self.yaw),-sin(self.pitch)*sin(self.yaw),cos(self.pitch)]])
        
        self.fire_to = 0
        self.alive = True
        
        self.camera.AddObject3d(self.object)
    
    def Update(self):
        if self.alive:
            if self.type in ['comm_sat']:
                self.pitch += 0.01
                self.object.DCM = np.array([[cos(self.pitch)*cos(self.yaw),-cos(self.pitch)*sin(self.yaw),-sin(self.pitch)],\
                    [sin(self.yaw),cos(self.yaw),0.0],[sin(self.pitch)*cos(self.yaw),-sin(self.pitch)*sin(self.yaw),cos(self.pitch)]])
            if self.type in ['defence_plat_a']:
                if self.fire_to == 0:
                    rel_pos = self.camera.pos-self.pos
                    dist = np.linalg.norm(rel_pos)
                    if dist < self.attack_range:
                        vel = 1.5*rel_pos/np.linalg.norm(rel_pos)
                        #print self.camera.pos, self.pos, rel_pos, vel
                        self.parent.projectiles.New('laser2',self.pos[:],vel)
                        self.fire_to = self.fire_rate
                        #resources.soundfx['laser2'].play()
                        resources.PlaySound('laser2')
                else:
                    self.fire_to -= 1
            if self.type in ['defence_plat_b']:
                if self.fire_to == 0:
                    rel_pos = self.camera.pos-self.pos
                    dist = np.linalg.norm(rel_pos)
                    if dist < self.attack_range:
                        vel = 2.0*rel_pos/np.linalg.norm(rel_pos)
                        #print self.camera.pos, self.pos, rel_pos, vel
                        self.parent.projectiles.New('laser3',self.pos[:],vel)
                        self.fire_to = self.fire_rate
                        #resources.soundfx['laser2'].play()
                        resources.PlaySound('laser2')
                else:
                    self.fire_to -= 1
            if self.type in ['interceptor']:
                rel_pos = self.camera.pos-self.pos
                dist = np.linalg.norm(rel_pos)
                DCM2 = np.array([[cos(self.yaw),-sin(self.yaw),0.0],\
                    [sin(self.yaw),cos(self.yaw),0.0],[0.0,0.0,1.0]])
                vertex_c = [np.dot(rel_pos,DCM2[0]),np.dot(rel_pos,DCM2[1]),np.dot(rel_pos,DCM2[2])]
                hordist = sqrt(vertex_c[0]*vertex_c[0] + vertex_c[1]*vertex_c[1])
                if hordist > 0.0:
                    azimuth = atan2(vertex_c[1],vertex_c[0])
                    elev = -atan(vertex_c[2]/hordist)
                else:
                    azimuth = 0.0
                    elev = 0.0
                
                # Evading
                dvx = 0
                self.maxv = 0.4
                if dist < self.viewrange:
                    if self.evade_to > 0: # evading
                        
                        azimuth = azimuth + pi/2
                        elev = 0.0
                        
                        # attitude
                        if self.pitch < elev:
                            self.pitch += min(0.06,(elev-self.pitch))
                        elif self.pitch > elev:
                            self.pitch -= min(0.06,(self.pitch-elev))
                        if self.pitch >= pi/2:
                            self.pitch = pi/2-0.0001
                        elif self.pitch <= -pi/2:
                            self.pitch = -pi/2+0.0001
                        
                        if azimuth > 0:
                            self.yaw -= min(0.06,fabs(azimuth))
                        elif azimuth < 0:
                            self.yaw += min(0.06,fabs(azimuth))
                        
                        self.object.DCM = np.array([[cos(self.pitch)*cos(self.yaw),-cos(self.pitch)*sin(self.yaw),-sin(self.pitch)],\
                            [sin(self.yaw),cos(self.yaw),0.0],[sin(self.pitch)*cos(self.yaw),-sin(self.pitch)*sin(self.yaw),cos(self.pitch)]])
                        
                        dvx = 2*self.accel
                        self.maxv = 0.8
                        self.evade_to -= 1
                        
                    else: # tracking to attack
                        
                        # attitude
                        if self.pitch < elev:
                            self.pitch += min(0.06,(elev-self.pitch))
                        elif self.pitch > elev:
                            self.pitch -= min(0.06,(self.pitch-elev))
                        if self.pitch >= pi/2:
                            self.pitch = pi/2-0.0001
                        elif self.pitch <= -pi/2:
                            self.pitch = -pi/2+0.0001
                        
                        if azimuth > 0:
                            self.yaw -= min(0.06,fabs(azimuth))
                        elif azimuth < 0:
                            self.yaw += min(0.06,fabs(azimuth))
                        
                        self.object.DCM = np.array([[cos(self.pitch)*cos(self.yaw),-cos(self.pitch)*sin(self.yaw),-sin(self.pitch)],\
                            [sin(self.yaw),cos(self.yaw),0.0],[sin(self.pitch)*cos(self.yaw),-sin(self.pitch)*sin(self.yaw),cos(self.pitch)]])
                        
                        # Movement
                        if dist > 20.0:
                            dvx = self.accel
                        else:
                            dvx = 0
                        
                        # check for fire
                        if self.fire_to == 0 and dist < self.attack_range:
                            vel = 2.0*rel_pos/np.linalg.norm(rel_pos)
                            #print self.camera.pos, self.pos, rel_pos, vel
                            self.parent.projectiles.New('laser3',self.pos[:],vel)
                            self.fire_to = self.fire_rate
                            #resources.soundfx['laser2'].play()
                            resources.PlaySound('laser2')
                        elif self.fire_to > 0:
                            self.fire_to -= 1
                
                # spacecraft inertia
                self.velocity[0] += self.object.DCM[0][0]*dvx
                self.velocity[1] += self.object.DCM[0][1]*dvx
                self.velocity[2] += self.object.DCM[0][2]*dvx
                
                totalv = sqrt(pow(self.velocity[0],2) + pow(self.velocity[1],2) + pow(self.velocity[2],2))
                if totalv > self.maxv:
                    self.velocity[0] = self.maxv*(self.velocity[0]/totalv)
                    self.velocity[1] = self.maxv*(self.velocity[1]/totalv)
                    self.velocity[2] = self.maxv*(self.velocity[2]/totalv)
                if dvx > 0 and totalv > 0:
                    self.velocity[0] -= self.driftdecel*(self.velocity[0]/totalv)
                    self.velocity[1] -= self.driftdecel*(self.velocity[1]/totalv)
                    self.velocity[2] -= self.driftdecel*(self.velocity[2]/totalv)
                    totalv = sqrt(pow(self.velocity[0],2) + pow(self.velocity[1],2) + pow(self.velocity[2],2))
                    if totalv < self.driftvel:
                        self.velocity[0] = self.driftvel*(self.velocity[0]/totalv)
                        self.velocity[1] = self.driftvel*(self.velocity[1]/totalv)
                        self.velocity[2] = self.driftvel*(self.velocity[2]/totalv)
                self.object.pos[0] += self.velocity[0]
                self.object.pos[1] += self.velocity[1]
                self.object.pos[2] += self.velocity[2]
                self.pos = [self.object.pos[0],self.object.pos[1],self.object.pos[2]]
                
                
            if self.object.glow > 0:
                self.object.glow -= 1
        
    def Hit(self,damage):
        if self.type == 'interceptor':
            self.evade_to = 60
        self.object.glow = 10
        self.health -= damage
        if self.health <= 0:
            self.alive = False
            self.camera.objects.remove(self.object)
            if self.type in ['transport']:
                self.parent.explosioneffects.NewExplosion(self.pos,self.vel,big=True)
            else:
                self.parent.explosioneffects.NewExplosion(self.pos,self.vel)
            #resources.soundfx['explode'].play()
            resources.PlaySound('explode')
    
class JumpGate(object):
    def __init__(self,parent,camera,pos,active=False):
        self.parent = parent
        self.camera = camera
        self.pos = pos
        self.radius = 10.0
        self.pitch = random.random()
        self.roll = 0.0
        self.yaw = 0.0
        self.object = model.Object3d(resources.model_jumpgate,self.pos)
        
        self.active = active
        self.offlinenotice_to = 0
        
        self.camera.AddObject3d(self.object)
    
    def Update(self):
        
        if self.active:
            self.pitch += 0.01
            self.object.DCM = np.array([[cos(self.pitch)*cos(self.yaw),-cos(self.pitch)*sin(self.yaw),-sin(self.pitch)],\
                [sin(self.yaw),cos(self.yaw),0.0],[sin(self.pitch)*cos(self.yaw),-sin(self.pitch)*sin(self.yaw),cos(self.pitch)]])
        
        # check for player proximity
        rel_pos = self.camera.pos-self.pos
        dist = np.linalg.norm(rel_pos)
        if not self.parent.warping_out and dist < 3.0:
            if self.active:
                self.parent.warping_out = True
                self.parent.warp_to = 90
                resources.StopVoice()
                resources.PlaySound('jump')
            elif self.offlinenotice_to == 0:
                self.offlinenotice_to = 60
                resources.PlaySound('jumpoffline')
        
        if self.offlinenotice_to > 0:
            self.offlinenotice_to -= 1
    